enchant_sys_command:
    type: command
    name: denchant
    debug: false
    description: A better enchantment command.<&nl>The denchant command will enchant your item with the enchantment specified, at the level you choose (or at level 1, if no level is specified.) Enchantments pull from properly registered enchantments, which are all vanilla enchantments, custom denizen enchantments, and enchantments from any plugin that registers correctly. Lore is also correctly added (or updated) on the item when required, and uses correct roman numeralization up to ~3000. The command will also let you know if the enchantment is out of range of vanilla, if the enchantment will function like expected on the item, or if the enchantment conflicts with an existing enchantment. The command will not halt if it encounters any of this, it just lets you know if you're trying to keep within vanilla enchantment bounds. The command also will not double enchant an item, and enchanting an item with a higher level or lower level enchant of the same enchant will raise/lower the enchant on the item appropriately. Lastly, when denchant is used on a regular book, it will create an enchantment book, rather than a regular book with an enchantment on it. This command will only run if the player running the command has operator permissions.
    usage: /denchant [<&lt>name<&gt>] ({1}/<&lt>amount<&gt>)
    aliases:
        - dchant
    tab completions:
        1: <server.enchantments.parse[name]>
        2: <util.list_numbers_to[<enchantment[<context.args.first>].max_level>].get[<enchantment[<context.args.first>].min_level>].to[last]>
    script:
        - define enchant <context.args.first.if_null[]>
        - define level   <context.args.get[2].if_null[1]>
        - define item    <player.item_in_hand.if_null[]>
        - if !<player.exists>:
            - narrate "<&[error]>A player is required to use this command."
        - else if !<player.is_op>:
            - narrate "<&[error]>I'm sorry, but you do not have permission to perform this command. Please contact the server administrators if you believe that this is in error."
        - else if <[item].advanced_matches[air]>:
            - narrate "<&[error]>You are not holding an item to enchant."
        - else if <context.args.size> > 2:
            - narrate "<&[error]>Too many arguments."
        - else if <context.args.size> < 1:
            - narrate "<&[error]>Too few arguments."
        - else if !<[enchant].as_enchantment.is_truthy>:
            - narrate "<&[hard_notice]><context.args.first> <&[error]>is not a valid enchantment."
        - else:
            - define existing   <[item].enchantment_map.get[<[enchant]>].if_null[0]>
            - define enchant    <[enchant].as_enchantment>
            - define lore       <[item].lore.if_null[<list>]>
            - define lore_line  <reset><gray><[enchant].full_name[<[existing]>]>
            - define lore_line "<reset><red>Curse of <[enchant].full_name[<[existing]>]>" if:<[enchant].is_curse>
            # Remove existing enchantment lore if it exists
            - define lore:<-:<[lore_line]> if:<[existing].equals[0].not>
            # If it's a custom denizen enchant...
            - if <[enchant].key.starts_with[denizen]>:
                - define lore_line  <reset><gray><[enchant].full_name[<[level]>]>
                - define lore_line "<reset><red>Curse of <[enchant].full_name[<[level]>]>" if:<[enchant].is_curse>
                # Add enchantment lore if denizen enchantment
                - define lore:->:<[lore_line]>
                - definemap context:
                    enchantment: <[enchant]>
                    level:       <[level]>
                    item:        <[item]>
                    lore:        <[lore]>
                - customevent id:command_enchant_item context:<[context]> save:save
                # Cancel if the custom event cancelled
                - determine cancelled if:<entry[save].was_cancelled>
                # Determinations should come in the form of a single map entry, as custom event only handles adding to a list of outputs
                - define determines <entry[save].determination_list.first.as_map.if_null[<map>]>
                # Redefine values with ones from determination, if any
                - define item        <[determines.item]>        if:<[determines.item].exists>
                - define lore        <[determines.lore]>        if:<[determines.lore].exists>
                - define level       <[determines.level]>       if:<[determines.level].exists>
                - define enchantment <[determines.enchantment]> if:<[determines.enchantment].exists>
            # Change material to an enchanted book, if the player was holding a regular book
            - inventory adjust slot:hand material:enchanted_book if:<[item].advanced_matches[book]>
            # Add enchants to item
            - inventory adjust slot:hand enchantments:<map[<[enchant]>=<[level]>]>
            # Add updated lore to item
            - inventory adjust slot:hand lore:<[lore]>
            # Redefine item after adjusting it
            - define item <player.item_in_hand>
            # Add all non compatible enchantments to a list
            - foreach <[item].enchantment_map.exclude[<[enchant].name>]> key:enchantment as:ilevel:
                # Add the enchantment formatted name with color stripped (vanilla enchantments have gray coloring attached, so we want to remove that)
                - define conflicts:->:<[enchantment].as_enchantment.full_name[<[ilevel]>].strip_color> if:<[enchant].is_compatible[<[enchantment]>].not>
            - define conflict <empty>
            - define loop     <[conflicts].if_null[<list>]>
            # Add formatting and coloring to conflicts
            - foreach <[loop]>:
                - define conflict "<[conflict].if_null[]><&[soft_notice]><[loop_index].equals[<[loop].size>].if_true[ and ].if_false[, ]><&[warning]><[value]>"
                # if it's the first loop, just set conflict to the first conflicting enchant
                - define conflict <[value]> if:<[loop_index].equals[1]>
            # Warning if the player has enchanted outside the min/max range of the enchant
            - narrate "<&[warning]>The level you enchanted at is outside the range normally obtainable in vanilla." if:<[enchant].min_level.is_more_than[<[level]>].or[<[level].is_more_than[<[enchant].max_level>]>]>
            # Warning if the player has put an enchantment on an item it does not work on. If the item is an enchanted book, then this warning will never happen, as enchanted_books are always a "valid" item.
            - narrate "<&[warning]>This enchantment does not work on this item." if:<[enchant].can_enchant[<[item]>].not.and[<[item].advanced_matches[*book].not>]>
            # Warining if any of the enchants already on the item conflict with the new enchant
            - narrate "<&[warning]>This enchantment conflicts with <&[soft_notice]><[conflict]><&[warning]>." if:<[conflict].equals[].not>
            - narrate "<&[success]>You successfully enchanted a(n) <&[soft_notice]><[item].material.translated_name><&[success]> with <&[important]><[enchant].full_name[<[level]>].strip_color><&[success]>!"
